@@ -4,21 +4,20 @@
 
 #include "ast/visitor.h"
 
-namespace bpftrace {
-namespace ast {
+namespace bpftrace::ast {
 
 class Printer : public Visitor<Printer> {
 public:
-  explicit Printer(ASTContext &ctx, std::ostream &out)
-      : Visitor<Printer>(ctx), out_(out)
+  explicit Printer(std::ostream &out) : out_(out)
   {
   }
 
-  void print();
-
   using Visitor<Printer>::visit;
   void visit(Integer &integer);
+  void visit(NegativeInteger &integer);
+  void visit(Boolean &boolean);
   void visit(PositionalParameter &param);
+  void visit(PositionalParameterCount &param);
   void visit(String &string);
   void visit(StackMode &mode);
   void visit(Identifier &identifier);
@@ -27,38 +26,45 @@ public:
   void visit(Sizeof &szof);
   void visit(Offsetof &offof);
   void visit(Map &map);
+  void visit(MapAddr &map_addr);
+  void visit(MapDeclStatement &decl);
   void visit(Variable &var);
+  void visit(VariableAddr &var_addr);
   void visit(Binop &binop);
   void visit(Unop &unop);
   void visit(Ternary &ternary);
   void visit(FieldAccess &acc);
   void visit(ArrayAccess &arr);
+  void visit(TupleAccess &acc);
+  void visit(MapAccess &acc);
   void visit(Cast &cast);
   void visit(Tuple &tuple);
   void visit(ExprStatement &expr);
   void visit(AssignMapStatement &assignment);
+  void visit(AssignScalarMapStatement &assignment);
   void visit(AssignVarStatement &assignment);
   void visit(AssignConfigVarStatement &assignment);
   void visit(VarDeclStatement &decl);
   void visit(If &if_node);
   void visit(Unroll &unroll);
   void visit(While &while_block);
+  void visit(Range &range);
   void visit(For &for_loop);
   void visit(Config &config);
   void visit(Jump &jump);
   void visit(Predicate &pred);
   void visit(AttachPoint &ap);
   void visit(Probe &probe);
+  void visit(SubprogArg &arg);
   void visit(Subprog &subprog);
+  void visit(Import &imp);
   void visit(Program &program);
 
-  int depth_ = -1;
-
 private:
   std::ostream &out_;
+  int depth_ = 0;
 
   std::string type(const SizedType &ty);
 };
 
-} // namespace ast
-} // namespace bpftrace
+} // namespace bpftrace::ast
