@@ -209,24 +209,33 @@ void ResourceAnalyser::visit(Call &call)
     if (call.func == "printf") {
       if (probe_ != nullptr &&
           single_provider_type_postsema(probe_) == ProbeType::iter) {
+        resources_.bpf_print_fmts_id_map[&call] =
+            resources_.bpf_print_fmts.size();
         resources_.bpf_print_fmts.emplace_back(fmtstr);
       } else {
+        resources_.printf_args_id_map[&call] = resources_.printf_args.size();
         resources_.printf_args.emplace_back(
             fmtstr, tuple->fields, PrintfSeverity::NONE, SourceInfo(call.loc));
       }
     } else if (call.func == "errorf") {
+      resources_.printf_args_id_map[&call] = resources_.printf_args.size();
       resources_.printf_args.emplace_back(
           fmtstr, tuple->fields, PrintfSeverity::ERROR, SourceInfo(call.loc));
     } else if (call.func == "debugf") {
+      resources_.bpf_print_fmts_id_map[&call] =
+          resources_.bpf_print_fmts.size();
       resources_.bpf_print_fmts.emplace_back(fmtstr);
     } else if (call.func == "system") {
+      resources_.system_args_id_map[&call] = resources_.system_args.size();
       resources_.system_args.emplace_back(fmtstr, tuple->fields);
     } else {
+      resources_.cat_args_id_map[&call] = resources_.cat_args.size();
       resources_.cat_args.emplace_back(fmtstr, tuple->fields);
     }
   } else if (call.func == "join") {
     auto delim = call.vargs.size() > 1 ? call.vargs.at(1).as<String>()->value
                                        : " ";
+    resources_.join_args_id_map[&call] = resources_.join_args.size();
     resources_.join_args.push_back(delim);
   } else if (call.func == "count" || call.func == "sum" || call.func == "min" ||
              call.func == "max" || call.func == "avg") {
@@ -310,16 +319,20 @@ void ResourceAnalyser::visit(Call &call)
       call.addError() << "Different tseries bounds in a single map unsupported";
     }
   } else if (call.func == "time") {
+    resources_.time_args_id_map[&call] = resources_.time_args.size();
     if (!call.vargs.empty())
       resources_.time_args.push_back(call.vargs.at(0).as<String>()->value);
     else
       resources_.time_args.emplace_back("%H:%M:%S\n");
   } else if (call.func == "strftime") {
+    resources_.strftime_args_id_map[&call] = resources_.strftime_args.size();
     resources_.strftime_args.push_back(call.vargs.at(0).as<String>()->value);
   } else if (call.func == "print") {
     constexpr auto nonmap_headroom = sizeof(AsyncEvent::PrintNonMap);
     auto &arg = call.vargs.at(0);
     if (!arg.is<Map>()) {
+      resources_.non_map_print_args_id_map[&call] =
+          resources_.non_map_print_args.size();
       resources_.non_map_print_args.push_back(arg.type());
       const size_t fmtstring_args_size = nonmap_headroom + arg.type().GetSize();
       if (exceeds_stack_limit(fmtstring_args_size)) {
@@ -328,12 +341,15 @@ void ResourceAnalyser::visit(Call &call)
       }
     }
   } else if (call.func == "cgroup_path") {
+    resources_.cgroup_path_args_id_map[&call] =
+        resources_.cgroup_path_args.size();
     if (call.vargs.size() > 1)
       resources_.cgroup_path_args.push_back(
           call.vargs.at(1).as<String>()->value);
     else
       resources_.cgroup_path_args.emplace_back("*");
   } else if (call.func == "skboutput") {
+    resources_.skboutput_args_id_map[&call] = resources_.skboutput_args_.size();
     const auto &file = call.vargs.at(0).as<String>()->value;
     const auto &offset = call.vargs.at(3).as<Integer>()->value;
 
