@@ -386,16 +386,6 @@ static bool btf_type_is_modifier(const struct btf_type *t)
   }
 }
 
-const struct btf_type *BTF::btf_type_skip_modifiers(const struct btf_type *t,
-                                                    const struct btf *btf)
-{
-  while (t && btf_type_is_modifier(t)) {
-    t = btf__type_by_id(btf, t->type);
-  }
-
-  return t;
-}
-
 __u32 BTF::get_type_tags(std::unordered_set<std::string> &tags,
                          const BTFId &btf_id) const
 {
@@ -411,15 +401,73 @@ __u32 BTF::get_type_tags(std::unordered_set<std::string> &tags,
   return id;
 }
 
+static bool is_anon_btf_typename(const std::string &name)
+{
+  return name.empty() || name == "(anon)";
+}
+
+BTF::BTFId BTF::parse_anon_btf_name(const std::string &fullname)
+{
+  unsigned int btf_id;
+
+  // First, find BTF_ANON_STRUCT_PREFIX
+  auto prefix_pos = fullname.find(BTF_ANON_STRUCT_PREFIX);
+  if (prefix_pos == std::string::npos)
+    return BTFId{ .btf = nullptr, .id = 0 };
+
+  // After BTF_ANON_STRUCT_PREFIX, should have <id>_<objname>
+  auto tail_pos = prefix_pos + BTF_ANON_STRUCT_PREFIX.length();
+  auto underscore_pos = fullname.find("_", tail_pos);
+  if (underscore_pos == std::string::npos)
+    return BTFId{ .btf = nullptr, .id = 0 };
+
+  std::string idstr = fullname.substr(tail_pos, underscore_pos - tail_pos);
+  std::string btf_obj_name = fullname.substr(underscore_pos + 1);
+
+  try {
+    btf_id = std::stoul(idstr);
+  } catch (const std::invalid_argument &e) {
+    return BTFId{ .btf = nullptr, .id = 0 };
+  }
+
+  for (const auto &btf_obj : btf_objects) {
+    if (btf_obj.name == btf_obj_name) {
+      struct btf *btf = btf__type_by_id(btf_obj.btf, btf_id) ? btf_obj.btf
+                                                             : nullptr;
+      return BTFId{ .btf = btf, .id = btf_id };
+    }
+  }
+
+  return BTFId{ .btf = nullptr, .id = 0 };
+}
+
+std::string BTF::create_anon_btf_name(BTFId &btf_id)
+{
+  for (const auto &btf_obj : btf_objects) {
+    if (btf_obj.btf == btf_id.btf) {
+      return std::string(BTF_ANON_STRUCT_PREFIX) + std::to_string(btf_id.id) +
+             "_" + btf_obj.name;
+    }
+  }
+  return "";
+}
+
 SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
 {
-  const struct btf_type *t = btf__type_by_id(btf_id.btf, btf_id.id);
+  const struct btf_type *t;
+  BTFId id;
+
+  id = btf_id;
+
+  while ((t = btf__type_by_id(id.btf, id.id))) {
+    if (!btf_type_is_modifier(t))
+      break;
+    id.id = t->type;
+  }
 
   if (!t)
     return CreateNone();
 
-  t = btf_type_skip_modifiers(t, btf_id.btf);
-
   auto stype = CreateNone();
 
   if (btf_is_int(t)) {
@@ -428,15 +476,24 @@ SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
   } else if (btf_is_enum(t)) {
     stype = CreateInteger(t->size * 8, false);
   } else if (btf_is_composite(t)) {
-    std::string cast = btf_str(btf_id.btf, t->name_off);
-    if (cast.empty() || cast == "(anon)")
-      return CreateNone();
-    std::string comp = btf_is_struct(t) ? "struct" : "union";
-    std::string name = comp + " " + cast;
+    bool is_anon = false;
+    std::string recprefix = btf_is_struct(t) ? "struct " : "union ";
+    std::string rname = btf_str(btf_id.btf, t->name_off);
 
-    stype = CreateRecord(name, bpftrace_->structs.LookupOrAdd(name, t->size));
+    if (is_anon_btf_typename(rname)) {
+      is_anon = true;
+      rname = create_anon_btf_name(id);
+      if (rname.empty())
+        return CreateNone();
+    }
+    auto name = recprefix + rname;
+
+    auto record = bpftrace_->structs.LookupOrAdd(name, t->size).lock();
+    stype = CreateRecord(name, record);
+    if (is_anon)
+      stype.SetAnon();
     if (resolve_structs)
-      resolve_fields(stype);
+      resolve_fields(id, std::move(record), 0);
   } else if (btf_is_ptr(t)) {
     const BTFId pointee_btf_id = { .btf = btf_id.btf, .id = t->type };
     std::unordered_set<std::string> tags;
@@ -449,13 +506,20 @@ SizedType BTF::get_stype(const BTFId &btf_id, bool resolve_structs)
     const auto &elem_type = get_stype(
         BTFId{ .btf = btf_id.btf, .id = array->type });
     if (elem_type.IsIntTy() && elem_type.GetSize() == 1) {
-      // Note that we allocate an extra element in this case, since it is
-      // necessary for the frontend to detect when a well-formed string is
-      // stored versus a non-well-formed string. We consider the array to be
-      // well-formed even if it does not contain a NUL-terminator, since it has
-      // a known size up front. However, our string must store the terminator
-      // to signal that it is well-formed.
-      stype = CreateString(array->nelems + 1);
+      // Auto convert char arrays to strings.
+      // This is the least worse option since users have come to expect
+      // strings when they call `print`, `printf` or do string literal
+      // comparison to this BTF type but note that we don't add 1
+      // to the length to ensure it's well formed because
+      // string.GetSize() will return len + 1, which is a mismatch with the
+      // actual type. So this is making the assumption that the string IS well
+      // formed (NULL terminated), which is not guaranteed. A string that is
+      // missing the NULL terminator at the end will appear as a truncated
+      // string, e.g. `...` as a trailing suffix. If this conversion is
+      // incorrect, because it's not an actual string, users have the option to
+      // cast all strings to int8 arrays, e.g.
+      // `(int8[])"mystring"`.
+      stype = CreateString(array->nelems);
     } else {
       stype = CreateArray(array->nelems, elem_type);
     }
@@ -503,9 +567,10 @@ std::shared_ptr<Struct> BTF::resolve_args(std::string_view func,
 
   const struct btf_param *p = btf_params(t);
   __u16 vlen = btf_vlen(t);
-  if (vlen >= arch::Host::arguments().size()) {
-    err = "functions with more than 6 parameters are "
-          "not supported.";
+  if (vlen > arch::Host::arguments().size()) {
+    err = "functions with more than " +
+          std::to_string(arch::Host::arguments().size()) +
+          " parameters are not supported.";
     return nullptr;
   }
 
@@ -582,7 +647,7 @@ std::string BTF::get_all_funcs_from_btf(const BTFObj &btf_obj) const
     if (bpftrace_ && !bpftrace_->is_traceable_func(func_name))
       continue;
 
-    if (btf_vlen(t) >= arch::Host::arguments().size())
+    if (btf_vlen(t) > arch::Host::arguments().size())
       continue;
 
     funcs += btf_obj.name + ":" + func_name + "\n";
@@ -623,7 +688,7 @@ std::string BTF::get_all_raw_tracepoints_from_btf(const BTFObj &btf_obj) const
       break;
     }
 
-    if (btf_vlen(t) >= arch::Host::arguments().size())
+    if (btf_vlen(t) > arch::Host::arguments().size())
       continue;
 
     bool found = false;
@@ -1122,19 +1187,26 @@ __s32 BTF::find_id_in_btf(struct btf *btf,
 
 void BTF::resolve_fields(const SizedType &type)
 {
+  BTFId type_id;
+
   if (!type.IsRecordTy())
     return;
 
-  auto record = bpftrace_->structs.Lookup(type.GetName()).lock();
+  auto const &name = type.GetName();
+  auto record = bpftrace_->structs.Lookup(name).lock();
   if (record->HasFields())
     return;
 
-  auto type_name = btf_type_str(type.GetName());
-  auto type_id = find_id(type_name, BTF_KIND_STRUCT);
-  if (!type_id.btf)
-    type_id = find_id(type_name, BTF_KIND_UNION);
-  if (!type_id.btf)
-    return;
+  if (type.IsAnonTy())
+    type_id = parse_anon_btf_name(name);
+  else {
+    __u32 kind = name.starts_with("struct") ? BTF_KIND_STRUCT : BTF_KIND_UNION;
+    auto type_name = btf_type_str(name);
+
+    type_id = find_id(type_name, kind);
+    if (!type_id.btf)
+      return;
+  }
 
   resolve_fields(type_id, std::move(record), 0);
 }
@@ -1175,8 +1247,7 @@ void BTF::resolve_fields(const BTFId &type_id,
     __u32 field_offset = start_offset +
                          (btf_member_bit_offset(btf_type, i) / 8);
 
-    if (btf_is_composite(field_type) &&
-        (field_name.empty() || field_name == "(anon)")) {
+    if (btf_is_composite(field_type) && is_anon_btf_typename(field_name)) {
       resolve_fields(field_id, record, field_offset);
       continue;
     }
