@@ -31,6 +31,10 @@ struct Foo4 {
   unsigned int d : 20;
 };
 
+enum FooEnum {
+  VALUE
+};
+
 struct Foo3 *func_1(int a,
                     struct Foo1 *foo1,
                     struct Foo2 *foo2,
@@ -80,10 +84,33 @@ struct ArrayWithCompoundData {
   struct Foo3 *data[2];
 };
 
+typedef struct {
+  int a;
+} AnonStructTypedef;
+
+AnonStructTypedef AnonTypedefArray[8];
+
+struct anon_structs {
+  AnonStructTypedef AnonTypedefArray[8];
+  struct {
+    int a;
+    int b;
+    struct {
+      int c;
+      int d;
+    } AnonSubArray[2];
+  } AnonArray[4];
+};
+
 void func_array_with_compound_data(struct ArrayWithCompoundData *arr)
 {
 }
 
+void func_anon_struct(struct anon_structs *AnonStruct,
+                      AnonStructTypedef *AnonTypedef)
+{
+}
+
 struct task_struct {
   int pid;
   int pgid;
@@ -163,6 +190,7 @@ int main(void)
   struct bpf_iter__task_vma iter_task_vma;
   struct bpf_map bpf_map;
   struct sock sk;
+  enum FooEnum e;
 
   func_1(0, 0, 0, 0, 0);
 
