@@ -8,6 +8,7 @@
 namespace bpftrace::test::resource_analyser {
 
 using ::testing::_;
+using ::testing::ElementsAre;
 
 void test(BPFtrace &bpftrace,
           const std::string &input,
@@ -117,4 +118,14 @@ TEST(resource_analyser, fmt_string_args_non_map_print_arr)
   EXPECT_EQ(resources.max_fmtstring_args_size, 40U + 1U);
 }
 
+TEST(resource_analyser, print_non_map_print_correct_args_order)
+{
+  RequiredResources resources;
+  test(R"(begin { print({ $x = 1; print("bob"); $x > 1}) })", true, &resources);
+
+  EXPECT_THAT(resources.non_map_print_args,
+              ElementsAre(SizedType(Type::string, 4),
+                          SizedType(Type::boolean, 1)));
+}
+
 } // namespace bpftrace::test::resource_analyser
